
set echo on

DECLARE

   CURSOR c_source IS
      SELECT *
      FROM   update_table;

   TYPE aat_whole_row IS TABLE OF c_source%ROWTYPE
      INDEX BY PLS_INTEGER;

   TYPE rt_update_cols IS RECORD
   ( name update_table.name%TYPE
   , type update_table.type%TYPE
   );

   TYPE aat_update_cols IS TABLE OF rt_update_cols
      INDEX BY PLS_INTEGER;

   TYPE aat_id IS TABLE OF update_table.id%TYPE
      INDEX BY PLS_INTEGER;

   aa_rows     aat_whole_row;    -- bulk fetch source data
   aa_ids      aat_id;           -- primary key array
   aa_updates  aat_update_cols;  -- update data array

BEGIN

   /* Fetch in one go as we are using small demo volumes... */
   OPEN c_source;
   FETCH c_source BULK COLLECT INTO aa_rows;
   CLOSE c_source;

   /* Process batch of data... */
   FOR i IN 1 .. aa_rows.COUNT LOOP

      --[complex processing goes here]--
      aa_rows(i).name := LOWER(aa_rows(i).name);
      aa_rows(i).type := LOWER(aa_rows(i).type);

      /* Assign IDs to PK array... */
      aa_ids(i)          := aa_rows(i).id;

      /* Assign updateable data to update array... */
      aa_updates(i).name := aa_rows(i).name;
      aa_updates(i).type := aa_rows(i).type;

   END LOOP;

   /* 
   || FORALL update using SET ROW.
   || This will not update primary key...
   */
   FORALL i IN 1 .. aa_rows.COUNT
      UPDATE ( SELECT name, type
               FROM   update_table
               WHERE  id = aa_ids(i) )
      SET    ROW = aa_updates(i);

   DBMS_OUTPUT.PUT_LINE( SQL%ROWCOUNT || ' rows updated.' );

END;
/


